#!/usr/bin/perl -w

# nagios plugin using openweathermap's JSON api
#
# (c) 2013 by Frederic Krueger / check-openweathermap@dev.techno.holics.at
#
# Licensed under Creative Commons Attribution-ShareAlike 3.0 Unported
# There is no warranty of any kind, explicit or implied, for anything this software does or does not do.
# 
# Updates for this piece of software could be available under the following URL:
#   http://dev.techno.holics.at/check_openweathermap/


## uses
use Data::Dumper;             # for debugging
use JSON qw( decode_json );
use LWP::Simple;

use lib "/usr/lib/nagios/plugins";
use lib "/usr/lib64/nagios/plugins";
use lib "/srv/nagios/libexec";
use utils qw (%ERRORS);


## globals
my $dflt_where = "London,uk";    # default
my $baseurl = "http://api.openweathermap.org/data/2.5/weather?units=metric&q=";  # location-based url

my ($PROG_NAME, $PROG_VERSION) = ("check_openweathermap", "0.0.1");


## subs
sub usage
{
  my $msg = shift;
  print "\n";
  print "usage: $0 <locationstring> <wantedinfo>\n";
  print "\n";
  print "    locationstring is usually 'townname,countrytld' (ie. Wuerzburg,de or London,uk)\n";
  print "\n";
  print "    wantedinfo is either 'all' or one of the following:\n";
  print "      sys-country,sys-sunrise,sys-sunset (in epoch time)\n";
  print "      weather-main (ie. 'Clouds'), weather-description (ie. 'Broken clouds'), clouds-all (ie. '68' percent)\n";
  print "      main-temp,main-humidity,main-pressure,main-temp_min,main-temp_max,wind-speed,wind-gust,wind-deg (just the stated)\n";
  print "      name (name of location), coord-lat (latitude of location), coord-lon (longitude of location), id (id of weatherstation)\n";
  print "\n";
  print "    perfdata is being created with all available data, all the time.\n";
  print "\n";
  ## calling example
  print "Example with nearly all parameters available (as of 2013-07-28) being used:\n";
  print "nagios\@comp:/srv/nagios/libexec/contrib# ./check_openweathermap.pl Wuerzburg,de sys-sunrise,sys-sunset,main-pressure,main-temp,main-humidity,wind-gust,wind-speed,wind-deg,weather-main,coord-lat,coord-lon,clouds-all,id,name
OK - sys-sunrise=1374983154, sys-sunset=1375038455, main-pressure=1012, main-temp=298.69, main-humidity=38, wind-gust=2.06, wind-speed=1.03, wind-deg=0, weather-main=Clouds, coord-lat=49.787781, coord-lon=9.93611, clouds-all=68, id=2805615, name=Wuerzburg | sys-sunrise=1374983154;;;; sys-sunset=1375038455;;;; main-pressure=1012;;;; main-temp=298.69;;;; main-humidity=38;;;; wind-gust=2.06;;;; wind-speed=1.03;;;; wind-deg=0;;;; coord-lat=49.787781;;;; coord-lon=9.93611;;;; clouds-all=68;;;; id=2805615;;;
nagios\@comp:/srv/nagios/libexec/contrib
";
  print "\n\n";
  ## nagios setup info
  print "## NAGIOS COMMANDS SETUP\n";
  print 'define command{
  command_name  check_openweathermap
  command_line  $USER1$/contrib/check_openweathermap.pl $ARG1$ $ARG2$ $ARG3$ $ARG4$ $ARG5$ $ARG6$ $ARG7$
}

';
  print "## NAGIOS SERVICE SETUP\n";
  print "define service{
  use                    local-service         ; or local-service-pnp, if you have pnp4nagios integrated
  host_name              yourhost
  service_description    Weather myplace
  check_command          check_openweathermap!London,uk!name,weather-main,main-temp,main-humidity,main-pressure,wind-speed,wind-deg,wind-gust,clouds-all
}

For the pnp4nagios related files (check_command entry as well as the graph-template), check the archive you got this script in.

";

  ## licensing info
  print "$PROG_NAME v$PROG_VERSION is licensed under Creative Commons Attribution-ShareAlike 3.0 Unported License.\n";
  print "There is no warranty of any kind, explicit or implied, for anything this software does or does not do.\n";
  print "\n";
  print "(c) 2013 by Frederic Krueger / check-openweathermap\@dev.techno.holics.at\n";
  print "\n";

  if ((defined($msg)) and ($msg eq ""))
  {
    print "\nERROR: $msg\n\n";
  }

  exit($ERRORS{"UNKNOWN"});
} # end sub usage


sub nagexit
{
  my ($nagstate, $msg, $perfdata, $debugout) = @_;
  $perfdata = ""  if (!defined($perfdata));
  $nagstate = "UNKNOWN" if ((!defined($nagstate)) or (!defined($ERRORS{uc($nagstate)})));
  $nagstate = uc($nagstate);
  $msg = $nagstate  if (!defined($msg));
  $debugout = ""  if (!defined($debugout));

  print "$msg" .($perfdata ne "" ? " | $perfdata" : "") ."\n";
  if ($debugout ne "") { print "$debugout\n"; }                 # add debug out, only used with errors

  exit ($ERRORS{$nagstate});
} # end sub nagexit



## args
if ((!defined($ARGV[0])) or (!defined($ARGV[1])))
{
  usage();
} # end if not enough arguments gotten

my $location   = $ARGV[0];
my $wantedinfo = $ARGV[1];

my $weatherinfo_json = get("${baseurl}$location");
if (!defined($weatherinfo_json))
{
  usage ("Problem getting weatherinfo for argument '$location' (url: $baseurl$location)\n$weatherinfo_json\n-------end-of-html-output-------");
}

# now JSON-parse the data gotten (hopefully)
my $wiref;
eval {
  $wiref = decode_json($weatherinfo_json);
};
if ( ($@) or ((!defined($wiref)) or (ref($wiref) ne "HASH")) )
{
  nagexit ("UNKNOWN", "UNKNOWN - Problem parsing returned JSON for location '$location'", "", "$@");
}

#print "dumping wiref: $wiref =>\n" .Dumper($wiref). "\n\n";

my $datastr = "";
my $perfdatastr = "";

if ($wantedinfo ne "")
{
  my @wantedinfos = split /,/, $wantedinfo;
  my $val = ""; my $key = "";
  foreach $inpkey (@wantedinfos)
  {
    $key = $inpkey;         $val = "";
    $key =~ s/-/'}{'/isg;   $key = "{'$key'}";
    $key =~ s/^{'weather'}/{'weather'}[0]/g;      # get first entry of weather-array, which is a hash
#    print "wantedinfo became $key\n";
    eval "\$val = \${\$wiref}$key;";
    if ($val ne "")
    {
#      print "key $key => val '$val'\n";
      $datastr .= "$inpkey=$val, ";
      if ($val =~ /^[0-9\.\-]+$/) { $perfdatastr .= "$inpkey=$val;;;; "; }   # add perfdata only if numeric data gotten
    }
  }
  if (length($datastr) > 0) { $datastr = substr($datastr, 0, length($datastr)-2); }
  if (length($perfdatastr) > 0) { $perfdatastr = substr($perfdatastr, 0, length($perfdatastr)-2); }
}

if ((length($datastr) > 0) and (length($perfdatastr) > 0))
{
  nagexit ("OK", "OK - $datastr", $perfdatastr, "");
}


# default exitcode:
nagexit ("UNKNOWN", "UNKNOWN - Unknown error occured for location '$location'", "", "");
